/*
    $Id: CLog.cc 79 2009-08-09 15:02:36Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>

#include "CLog.h"


int CLog::loglevel = 1;
bool CLog::isInit = false;
pthread_mutex_t CLog::mutex;

void CLog::init(int loglevel){
    if(isInit)
        return;
    pthread_mutex_init(&mutex, NULL);
    CLog::loglevel = loglevel;
    isInit = true;
}

void CLog::Log(int level, const char *format, ...) {
    if(!CLog::isInit)
        return;
    if (level > CLog::loglevel)
        return;

    time_t t;
    char cnow[200];
    struct tm tmp;

    pthread_mutex_lock(&mutex);
    pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, &mutex);

    time(&t);
    localtime_r(&t, &tmp);
    if (strftime(cnow, sizeof(cnow), STRFTIME_FORMAT, &tmp) == 0) {
        fprintf(stderr, "strftime returned 0");
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "%s: ", cnow);

    va_list ap;
    va_start(ap, format);
    vfprintf(stdout, format, ap);
    va_end(ap);

    fprintf(stdout,"\n");
    fflush(stdout);

    //pthread_mutex_unlock(&mutex);
    pthread_cleanup_pop(1);
}

void CLog::Log(int level, int err, const char *format, ...) {
    if(!isInit)
        return;
    if (level > loglevel)
        return;

    time_t t;
    char cnow[200], errmsg[200];
    struct tm tmp;

    pthread_mutex_lock(&mutex);
    pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, &mutex);

    time(&t);
    localtime_r(&t, &tmp);
    if (strftime(cnow, sizeof(cnow), STRFTIME_FORMAT, &tmp) == 0) {
        fprintf(stderr, "strftime returned 0");
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "%s: ", cnow);

    va_list ap;
    va_start(ap, format);
    vfprintf(stdout, format, ap);
    va_end(ap);

    strerror_r(err, errmsg, 200);
    fprintf(stdout,": %s", errmsg);
    fprintf(stdout,"\n");
    fflush(stdout);

    //pthread_mutex_unlock(&mutex);
    pthread_cleanup_pop(1);
}

void CLog::LogHex(int level, const char *msg, const uchar *hex, int size) {
    if(!isInit)
        return;
    if (level > loglevel)
        return;

    time_t t;
    char cnow[200];
    struct tm tmp;

    pthread_mutex_lock(&mutex);
    pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, &mutex);

    time(&t);
    localtime_r(&t, &tmp);
    if (strftime(cnow, sizeof(cnow), STRFTIME_FORMAT, &tmp) == 0) {
        fprintf(stderr, "strftime returned 0");
        exit(EXIT_FAILURE);
    }
    fprintf(stdout, "%s: ", cnow);

    fprintf(stdout, "%s:", msg);
    for (int i=0; i < size; i++)
        fprintf(stdout, " %0x", hex[i]);
    fprintf(stdout, "\n");
    fflush(stdout);

    //pthread_mutex_unlock(&mutex);
    pthread_cleanup_pop(1);
}

