/*
    $Id: CSkiputler.cc 75 2009-05-27 21:18:31Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <string.h>

#include "CLog.h"
#include "CSkiputler.h"
#include "CSkypeAPIClient.h"
#include "CButlerPhoneClient.h"

CSkiputler::CSkiputler(void) {
    this->state=SSInit;
    this->flags=SFNone;
}

CSkiputler::~CSkiputler(void) {
    //delete skypeapi;
    //delete butlerphone;
}

void CSkiputler::terminate(void) {
    delete skypeapi;
    delete butlerphone;
}

CSkiputler::skiputlerState_t CSkiputler::setState(skiputlerState_t state) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::setState: new=%u old=%u", state, this->state);

    skiputlerState_t oldState = this->state;
    this->state = state;
    return oldState;
}

void CSkiputler::start(void) {
    butlerphone = new CButlerPhone(this);
    butlerphone->start();
    skypeapi = new CSkypeAPI(this);
    skypeapi->start();
}

// client methods
void CSkiputler::onSkypeCall(CSkypeAPI::callStat_t status, CSkypeAPI::callType_t type, const char *dialString) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onSkypeCall(%u, %u, \"%s\"", status, type, dialString);

    if (type == CSkypeAPI::callType_UNDEF) {
        // TODO
        CLog::Log(LOGLEVEL_ERROR, "CSkiputler::onSkypeCall: callType_UNDEF");
        return;
    }

    if (type < CSkypeAPI::callType_OUTGOING_PSTN) {
    // Incoming calls
        switch (status) {
            case CSkypeAPI::callStat_RINGING :
                CLog::Log(LOGLEVEL_ERROR, "CSkiputler::onSkypeCall: Incoming RINGING");
                if (this->state == CSkiputler::SSIdle) {
                    butlerphone->ring(dialString, strlen(dialString));
                    setState(SSRinging);
                }
                break;

            case CSkypeAPI::callStat_INPROGRESS :
                CLog::Log(LOGLEVEL_ERROR, "CSkiputler::onSkypeCall: Incoming INPROGRESS");
                if (this->state == SSRinging) {          
                    butlerphone->stopRing();
                    setState(SSOnline);
                }
                break;                

            case CSkypeAPI::callStat_CANCELLED :
                CLog::Log(LOGLEVEL_ERROR, "CSkiputler::onSkypeCall: Incoming CANCELLED");
                if (this->state == SSRinging) {          
                    butlerphone->stopRing();
                    setState(SSIdle);
                }
                break;

            default:
                if (this->state == SSRinging) {          
                    butlerphone->stopRing();
                    setState(SSIdle);
                } 
        }
    }
    else {
    // Outgoing calls
        switch (status) {
            case CSkypeAPI::callStat_CANCELLED :
                CLog::Log(LOGLEVEL_ERROR, "CSkiputler::onSkypeCall: Outgoing CANCELLED");
                if (this->state == SSOnline) {
                    setState(SSIdle);
                    butlerphone->hangup();
                }
                break;

            case CSkypeAPI::callStat_RINGING :
            case CSkypeAPI::callStat_ROUTING :
                CLog::Log(LOGLEVEL_ERROR, "CSkiputler::onSkypeCall: Outgoing RINGING");
                if (this->state == SSIdle) {
                    setState(SSOnline);
                }
                break;

            case CSkypeAPI::callStat_FINISHED :
                CLog::Log(LOGLEVEL_ERROR, "CSkiputler::onSkypeCall: Outgoing FINISHED");
                if (this->state == SSOnline) {
                    setState(SSIdle);
                    butlerphone->hangup();
                }
                break;

            default : break;
        }
    }
}

void CSkiputler::onSkypeOnline(void) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onSkypeOnline");
    if (flagIsSet(SFSkypeOnline))
        return;

    setFlag(SFSkypeOnline);   

    if (flagIsSet(SFSkypeOnline) && flagIsSet(SFPhoneOnline)) {
        butlerphone->setButlerStatus(CButlerPhone::BSOnline);	
        setState(SSIdle);
    }
}

void CSkiputler::onSkypeOffline(void) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onSkypeOffline");
    if (!flagIsSet(SFSkypeOnline))
        return;

    clearFlag(SFSkypeOnline);

    if (flagIsSet(SFPhoneOnline)) {
        butlerphone->setButlerStatus(CButlerPhone::BSOffline);
    }   

    setState(SSOffline);
}

void CSkiputler::onButlerConnected(void) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerConnected");
    if (flagIsSet(SFPhoneOnline))
        return;

    setFlag(SFPhoneOnline);
  
    if (flagIsSet(SFSkypeOnline) && flagIsSet(SFPhoneOnline)) {
	butlerphone->setButlerStatus(CButlerPhone::BSOnline);	
        setState(SSIdle);
    }
    else
        butlerphone->setButlerStatus(CButlerPhone::BSOffline);	
}

void CSkiputler::onButlerDisconnected(void) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerDisconnected");
    if (!flagIsSet(SFPhoneOnline))
        return;

    clearFlag(SFPhoneOnline);
  
    setState(SSOffline);
}

void CSkiputler::onButlerAcceptCall(void) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerAcceptCall");
    if(this->state != SSRinging)
        return;
    skypeapi->acceptCall();
}

void CSkiputler::onButlerDialNumber(char *dialString) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerDialNumber: '%s'", dialString);
    skypeapi->initiateCall(dialString);
}

void CSkiputler::onButlerDialName(char *dialString) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerDialName: '%s'", dialString);
    skypeapi->initiateCallByName(dialString);
}

void CSkiputler::onButlerHangup(void) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerHangup");
    if(this->state != SSOnline)
        return;
    skypeapi->hangupCall();
    setState(SSIdle);
}

void CSkiputler::onButlerKeyContacts(CButlerPhone::contactKey_t key) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerKeyContacts");
    const char *contact;

    switch (key) {

        case CButlerPhone::CKFirst :
            CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerKeyContacts: CKFirst");
            if (contact = skypeapi->getContactsFirst())
                butlerphone->displayContact(contact, strlen(contact));
            break;
        case CButlerPhone::CKUp :
            CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerKeyContacts: CKUp");
            if (contact = skypeapi->getContactsNext())
                butlerphone->displayContact(contact, strlen(contact));
            break;
        case CButlerPhone::CKDown :
            CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerKeyContacts: CKDown");
            if(contact = skypeapi->getContactsPrevious())
                butlerphone->displayContact(contact, strlen(contact));
            break;
        case CButlerPhone::CKExit :
            CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerKeyContacts: CKExit");
            break;
    }
}

void CSkiputler::onButlerKeyCalls(CButlerPhone::contactKey_t key) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerKeyCalls");
    const char *contact;
    CSkypeAPI::callsList_struct *data;

    switch (key) {

        case CButlerPhone::CKFirst :
            CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerKeyCalls: CKFirst");
            if (data = skypeapi->getCallsFirst())
                butlerphone->displayCall(data->handle.c_str(), data->handle.length(), data->online, data->timestamp);
            break;
        case CButlerPhone::CKUp :
            CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerKeyCalls: CKUp");
            if (data = skypeapi->getCallsFirst())
                butlerphone->displayCall(data->handle.c_str(), data->handle.length(), data->online, data->timestamp);
            break;
        case CButlerPhone::CKDown :
            CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerKeyCalls: CKDown");
            if (data = skypeapi->getCallsFirst())
                butlerphone->displayCall(data->handle.c_str(), data->handle.length(), data->online, data->timestamp);
            break;
        case CButlerPhone::CKExit :
            CLog::Log(LOGLEVEL_DEBUG, "CSkiputler::onButlerKeyCalls: CKExit");
            break;
    }
}
