/*
    $Id: CSkiputler.h 75 2009-05-27 21:18:31Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "CButlerPhone.h"
#include "CSkypeAPI.h"

#include "CButlerPhoneClient.h"
#include "CSkypeAPIClient.h"

class CSkypeAPIClient;
class CButlerPhoneClient;

class CSkiputler : public CSkypeAPIClient, public CButlerPhoneClient {
    typedef enum {
        SSInit,
        SSOffline,
        SSIdle,
        SSRinging,
        SSOnline
    } skiputlerState_t;

    typedef enum {
        SFNone=0,
        SFPhoneOnline=1,
        SFSkypeOnline=2
    } skiputlerFlag_t;

    CButlerPhone *butlerphone;
    CSkypeAPI *skypeapi;
    skiputlerState_t state;
    skiputlerFlag_t flags;

    skiputlerState_t setState(skiputlerState_t state);

    void setFlag(skiputlerFlag_t flag) { this->flags = (skiputlerFlag_t)(this->flags | flag); };
    void clearFlag(skiputlerFlag_t flag) { this->flags = (skiputlerFlag_t)(this->flags & (~flag)); };
    bool flagIsSet(skiputlerFlag_t flag) { return ( (this->flags & flag) == flag); };

public:
    CSkiputler(void);
    ~CSkiputler(void);
    void terminate(void);
    void start(void);
    void onSkypeCall(CSkypeAPI::callStat_t status, CSkypeAPI::callType_t type, const char *dialString);
    void onSkypeOnline(void);
    void onSkypeOffline(void);
    void onButlerConnected(void);
    void onButlerDisconnected(void);
    void onButlerAcceptCall(void);
    void onButlerDialNumber(char *dialString);
    void onButlerDialName(char *dialString);
    void onButlerHangup(void);
    void onButlerKeyContacts(CButlerPhone::contactKey_t key);
    void onButlerKeyCalls(CButlerPhone::contactKey_t key);
};
