/*
    $Id: CRingerThread.cc 60 2009-04-05 18:32:06Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <pthread.h>
#include <errno.h>

#include "CLog.h"
#include "CButlerPhone.h"

CButlerPhone::CRingerThread::CRingerThread(CButlerPhone *owner) {
    this->owner = owner;

    if (0 != pthread_create( &thread, NULL, threadfunc, this))
        CLog::Log(LOGLEVEL_ERROR, errno, "CRingerThread: Create StatusThread failed");
}

void CButlerPhone::CRingerThread::syncProc(void) {
    CLog::Log(LOGLEVEL_DEBUG, "CRingerThread::syncProc");
    owner->command((uchar[]){0x80, 0x01}, 2);
}

void *CButlerPhone::CRingerThread::threadfunc(void *ptr) {
    CLog::Log(LOGLEVEL_DEBUG, "CRingerThread::threadfunc");
    return ((CRingerThread*)ptr)->execute();
}

void *CButlerPhone::CRingerThread::execute(void) {
    CLog::Log(LOGLEVEL_DEBUG, "CRingerThread::execute");
    pthread_mutex_t *synchro_mutex = owner->getMutex();

    while(1) {
        pthread_mutex_lock(synchro_mutex);
        pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, synchro_mutex);
        syncProc();
        //pthread_mutex_unlock(synchro_mutex);
        pthread_cleanup_pop(1);
        pthread_testcancel();
        sleep(3);
    }
    return NULL;
}

void CButlerPhone::CRingerThread::terminate(void) {
    CLog::Log(LOGLEVEL_DEBUG, "CRingerThread::terminate");
    pthread_cancel(thread);
    CLog::Log(LOGLEVEL_DEBUG, "CRingerThread: canceled");
    pthread_join(thread, NULL);
    CLog::Log(LOGLEVEL_DEBUG, "CConnectThread: terminated");
}
