/*
    $Id: CSendFifo.cc 54 2009-03-30 23:44:18Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <deque>
#include <string.h>
#include <pthread.h>
#include "CButlerPhone.h"

CButlerPhone::CSendFifo::CSendFifo(void) {
    pthread_mutex_init(&critical_mutex, NULL);
}

void CButlerPhone::CSendFifo::push(const uchar *string, int size) {
    if (size < 1)
        return;
    uchar *buf = new uchar[size];
    memcpy(buf, string, size*sizeof(uchar));
    fifoData_t data;
    data.size=size;
    data.data = buf;

    pthread_mutex_lock(&critical_mutex);
    pipe.push_back(data);
    pthread_mutex_unlock(&critical_mutex);    

    if(onPush) {
        onPush(onPush_arg);
    }
        
}

CButlerPhone::CSendFifo::fifoData_t *CButlerPhone::CSendFifo::pop(void) {
    fifoData_t *data = new fifoData_t;
    pthread_mutex_lock(&critical_mutex);

    if (pipe.empty()) {
        pthread_mutex_unlock(&critical_mutex);
        delete data;
        return NULL;
    }

    *data = pipe.front();
    pipe.pop_front();
    pthread_mutex_unlock(&critical_mutex);

    return data;
}

bool CButlerPhone::CSendFifo::isEmpty(void) {
    bool ret;

    pthread_mutex_lock(&critical_mutex);
    ret = pipe.empty();
    pthread_mutex_unlock(&critical_mutex);    

    return ret;
}

void CButlerPhone::CSendFifo::clear(void) {
    pipe.clear();
}

CButlerPhone::CSendFifo::~CSendFifo(void) {
    pipe.clear();
    pthread_mutex_destroy(&critical_mutex);
}

void CButlerPhone::CSendFifo::setHandlerPush(t_NotifyEvent handler, void *arg) {
    this->onPush = handler;
    this->onPush_arg = arg;
}
