/*
    $Id: main.c 87 2009-08-31 15:21:37Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <Skype/Skype.h>
#include <Carbon/Carbon.h>
#include <glib.h>
#include <dbus/dbus-glib.h>
#include <unistd.h>

#include "skypeapi-dbusobj.h"


#define DBUS_SERVICE_SKYPE      "com.Skype.API"
#define DBUS_PATH_SKYPE_CLIENT  "/com/Skype/Client"
#define DBUS_PATH_SKYPE         "/com/Skype"
#define DBUS_INTERFACE_SKYPE    "com.Skype.API"
#define DBUS_INTERFACE_SKYPE_CLIENT "com.Skype.API.Client"



Boolean THREADED = TRUE;
DBusGConnection *connection = NULL;
DBusGProxy *proxy_skype = NULL;

struct SkypeDelegate mySkypeDelegate;

void SkypeNotificationReceived(CFStringRef aNotificationString) {
    GError *error = NULL;

#ifdef DEBUG
    printf("SkypeNotificationReceived() start \r\n");
	CFShow(aNotificationString);
#endif
    
    if (proxy_skype == NULL)
        return;

    int size = CFStringGetLength(aNotificationString);
    char *message = malloc((size+1) * sizeof(char));

    CFStringGetCString (aNotificationString, message, size+1, kCFStringEncodingMacRoman);

    if (!dbus_g_proxy_call(proxy_skype,
                         /* Method name to call. */
                         "Notify",
                         /* Where to store the GError. */
                         &error,
                         /* Parameter type of argument 0. Note that
                            since we're dealing with GLib/D-Bus
                            wrappers, you will need to find a suitable
                            GType instead of using the "native" D-Bus
                            type codes. */
                         G_TYPE_STRING,
                         /* Data of argument 0. In our case, this is
                            the well-known name for our server
                            example ("org.maemo.Platdev_ex"). */
                         message,
                         /* Input arguments are terminated with a
                            special GType marker. */
                         G_TYPE_INVALID,
                         /* Terminate list of return values. */
                         G_TYPE_INVALID)) {
        printf("Skype.Notify RPC failed: %s\n", error->message);
    /* Note that the whole call failed, not "just" the name
       registration (we deal with that below). This means that
       something bad probably has happened and there's not much we can
       do (hence program termination). */
    }    
}

void SkypeAttachResponse(unsigned int aAttachResponseCode){
	//printf("SkypeAttachResponse(int) start\r\n");
	printf("aAttachResponseCode = %i \r\n",aAttachResponseCode);
	if (aAttachResponseCode == 1) {
		//SendSkypeCommand(CFSTR("CALL echo123"));
	}
	//printf("SkypeAttachResponse(int) end\r\n");
}

void SkypeBecameAvailable(CFPropertyListRef aNotification){
    puts("Skype became available");
#ifdef DEBUG
	printf("SkypeBecameAvailable() start \r\n");
	printf(" aNotification ->%p<- \r\n",aNotification);
	printf("SkypeBecameAvailable() end \r\n");
#endif
}

void SkypeBecameUnavailable(CFPropertyListRef aNotification){
    puts("Skype became unavailable");
#ifdef DEBUG
	printf("SkypeBecameUnavailable() start \r\n");
	printf(" aNotification ->%p<- \r\n",aNotification);
	printf("SkypeBecameUnavailable() end \r\n");
#endif
} 

OSStatus init(void *data) {
#ifdef DEBUG
	printf("init() start \r\n");
#endif
	//struct SkypeDelegate mySkypeDelegate;
	mySkypeDelegate.SkypeNotificationReceived = SkypeNotificationReceived;
	mySkypeDelegate.SkypeAttachResponse = SkypeAttachResponse;
	mySkypeDelegate.SkypeBecameAvailable = SkypeBecameAvailable;
	mySkypeDelegate.SkypeBecameUnavailable = SkypeBecameUnavailable;
	/*
	mySkypeDelegate.SkypeNotificationReceived = NULL;
	mySkypeDelegate.SkypeAttachResponse = NULL;
	mySkypeDelegate.SkypeBecameAvailable = NULL;
	mySkypeDelegate.SkypeBecameUnavailable = NULL;
	*/
	mySkypeDelegate.clientApplicationName = CFSTR("Skiputler");

	SetSkypeDelegate(&mySkypeDelegate);
#ifdef DEBUG	
	printf("init() end \r\n");
#endif
	return 0;
}

OSStatus dbus_execute(void *data) {
    GMainLoop *mainloop;
    SkypeAPIDBusObj *dbusobj;

    /* Create a main loop that will dispatch callbacks. */
    mainloop = g_main_loop_new (NULL, FALSE);
    if (mainloop == NULL) {
        /* Print error and terminate. */
        puts("Couldn't create GMainLoop.");
        return(-1);
    }

    // Create one instance
    dbusobj = (SkypeAPIDBusObj*)g_object_new(SKYPEAPI_TYPE_DBUSOBJ, NULL);
    if(!dbusobj) {
        // Print error and terminate.
        puts("Failed to create SkypeAPIDBusObj instance.");
        return(-1);
    }

    // Register the instance to D-Bus
    dbus_g_connection_register_g_object(connection, DBUS_PATH_SKYPE, G_OBJECT (dbusobj));

    puts(DBUS_PATH_SKYPE " method 'Invoke' registered.");

    // Run the program
    g_main_loop_run(mainloop);
    puts(" dbus_execute return()");
    return(0);
}


int main(int argc, char* argv[])
{
    g_type_init();
    g_thread_init(NULL);

    DBusGProxy *proxy;
    GError *error = NULL;

    OSStatus		err;
    
    connection = dbus_g_bus_get(DBUS_BUS_SESSION, &error);
    if (connection == NULL) {
        printf("Failed to open connection to bus: %s", error->message);
        g_error_free (error);
        exit(-1);
    }
    
    proxy = dbus_g_proxy_new_for_name(connection,
                                       DBUS_SERVICE_DBUS,
                                       DBUS_PATH_DBUS,
                                       DBUS_INTERFACE_DBUS);

    if (proxy == NULL ) {
        puts("Failed to get a proxy for D-Bus.");
        exit(-1);
    }
    
    error = NULL;
    
    /* Will store the result of the RequestName RPC here. */
    guint result;
    
    if (!dbus_g_proxy_call(proxy,
                         /* Method name to call. */
                         "RequestName",
                         /* Where to store the GError. */
                         &error,
                         /* Parameter type of argument 0. Note that
                            since we're dealing with GLib/D-Bus
                            wrappers, you will need to find a suitable
                            GType instead of using the "native" D-Bus
                            type codes. */
                         G_TYPE_STRING,
                         /* Data of argument 0. In our case, this is
                            the well-known name for our server
                            example ("org.maemo.Platdev_ex"). */
                         DBUS_SERVICE_SKYPE,
                         /* Parameter type of argument 1. */
                         G_TYPE_UINT,
                         /* Data of argument 0. This is the "flags"
                            argument of the "RequestName" method which
                            can be use to specify what the bus service
                            should do when the name already exists on
                            the bus. We'll go with defaults. */
                         0,
                         /* Input arguments are terminated with a
                            special GType marker. */
                         G_TYPE_INVALID,
                         /* Parameter type of return value 0.
                            For "RequestName" it is UINT32 so we pick
                            the GType that maps into UINT32 in the
                            wrappers. */
                         G_TYPE_UINT,
                         /* Data of return value 0. These will always
                            be pointers to the locations where the
                            proxy can copy the results. */
                         &result,
                         /* Terminate list of return values. */
                         G_TYPE_INVALID)) {
    printf("D-Bus.RequestName RPC failed: %s\n", error->message);
    /* Note that the whole call failed, not "just" the name
       registration (we deal with that below). This means that
       something bad probably has happened and there's not much we can
       do (hence program termination). */
  }
#ifdef DEBUG
  /* Check the result code of the registration RPC. */
  g_print("RequestName returned %d.\n", result);
#endif
  if (result != 1) {
        puts("Failed to get the primary well-known name. RequestName result != 1");
    /* In this case we could also continue instead of terminating.
       We could retry the RPC later. Or "lurk" on the bus waiting for
       someone to tell us what to do. If we would be publishing
       multiple services and/or interfaces, it even might make sense
       to continue with the rest anyway.

       In our simple program, we terminate. Not much left to do for
       this poor program if the clients won't be able to find the
       Value object using the well-known name. */
  }


	while (!IsSkypeRunning()) {
		sleep(1);
	}
	
	init(NULL);
	
	ConnectToSkype();
/*
	MPTaskID        mpTaskID;
	OSStatus returnErr = MPCreateTask(&dbus_execute, (void *)connection , 0 , NULL, NULL, NULL, 0, &mpTaskID);
	if ( returnErr != noErr )                                              //  Alert if an error occured
    {
        printf("MPCreateTask returned an error\r\n");
    }
*/

    GMainLoop *mainloop;
    SkypeAPIDBusObj *dbusobj;
    //DBusGConnection *connection = (DBusGConnection *)data;

    // Create a main loop that will dispatch callbacks.
    mainloop = g_main_loop_new (NULL, FALSE);
    if (mainloop == NULL) {
        // Print error and terminate.
        puts("Couldn't create GMainLoop.");
        return(-1);
    }

    // Create one instance
    dbusobj = (SkypeAPIDBusObj*)g_object_new(SKYPEAPI_TYPE_DBUSOBJ, NULL);
    if(!dbusobj) {
        // Print error and terminate.
        puts("Failed to create SkypeAPIDBusObj instance.");
        return(-1);
    }

    //Register the instance to D-Bus
    dbus_g_connection_register_g_object(connection, DBUS_PATH_SKYPE, G_OBJECT (dbusobj));

    puts(DBUS_PATH_SKYPE " method 'Invoke' registered.");

    // Run the program
    //g_main_loop_run(mainloop);
    //puts(" dbus_execute return()");


	while (1) {
        g_main_context_iteration(NULL, FALSE);
		RunCurrentEventLoop(0.0001);
		//usleep(100);
		//SendSkypeCommand(CFSTR("CALL echo123"));

	}
	
	DisconnectFromSkype();
	return err;
}

