/*
    $Id: skypeapi-dbusobj.c 87 2009-08-31 15:21:37Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <Skype/Skype.h>
#include <dbus/dbus-glib-lowlevel.h>

#include "skypeapi-dbusobj.h"
#include "Invoke-method.h"

#define DBUS_PATH_SKYPE_CLIENT  "/com/Skype/Client"
#define DBUS_INTERFACE_SKYPE_CLIENT "com.Skype.API.Client"

extern DBusGConnection *connection;
extern DBusGProxy *proxy_skype;

G_DEFINE_TYPE(SkypeAPIDBusObj, skypeapi_dbusobj, G_TYPE_OBJECT)


/*
 * Object initializer
 */
void skypeapi_dbusobj_init (SkypeAPIDBusObj *obj) {
    g_assert(obj != NULL);
    obj->str_ret = NULL;	
}

/*
 * Class initializer
 */
void skypeapi_dbusobj_class_init (SkypeAPIDBusObjClass *klass) {
    g_assert(klass != NULL);

    /*Bind this GType into the Glib/D-Bus wrappers*/
    dbus_g_object_type_install_info (SKYPEAPI_TYPE_DBUSOBJ, &dbus_glib_skypeapi_dbusobj_object_info);
}

/*
 * Function that gets executed on "invoke".
 */
void skypeapi_dbusobj_invoke(SkypeAPIDBusObj *obj, const char *message, DBusGMethodInvocation *context) {
    g_assert(obj != NULL);

    int size;
    gchar *ret;

    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

    gchar *sender = (gchar *)dbus_g_method_get_sender(context);
    dbus_g_method_get_sender(context);

#ifdef DEBUG
    puts("INVOKE called!!");
    printf("Sender = %s\n", sender);
#endif

    if ((connection != NULL) && (proxy_skype == NULL)) {

        proxy_skype = dbus_g_proxy_new_for_name(connection,
                                       sender,
                                       DBUS_PATH_SKYPE_CLIENT,
                                       DBUS_INTERFACE_SKYPE_CLIENT);

        if (proxy_skype == NULL ) {
            puts("Failed to get a proxy for skiputler.");
            exit(-1);
        }

        puts("Connected to skiputler.");
    }

    CFStringRef answer = NULL;    
    CFStringRef command = CFStringCreateWithCString(NULL, message, kCFStringEncodingISOLatin1);
#ifdef DEBUG
    CFShow(command);
#endif

    if (CFStringHasPrefix (command, CFSTR("NAME ")) ) {

        answer = CFStringCreateCopy(NULL, CFSTR("OK"));
        size = 2;

    } else {

        answer = SendSkypeCommand(command);

        CFRelease(command);

        if (answer == NULL) {
            puts("answer = NULL");
            
            dbus_g_method_return(context, NULL);
            return;
        }

        size = CFStringGetLength(answer);
        size = CFStringGetMaximumSizeForEncoding (size, kCFStringEncodingUTF8);

#ifdef DEBUG
        CFShow(answer);
        printf ("Size = %d\n", size);
#endif
    }

    if (size > 0) {

        ret = malloc((size+1) * sizeof(gchar));

        if (!CFStringGetCString (answer, ret, size+1, kCFStringEncodingUTF8)) {
            puts("Problem converting CFStringGetCString");
            ret = NULL;
        }
    }
    else {
        ret = NULL;
    }

    CFRelease(answer);

#ifdef DEBUG
    printf("Command = %s\n", message);
    printf("Answer = %s\n", ret);

    //int i=0;
    //while(ret[i] != '\0') {
    //    printf("ret[%d] = %c\n", i, ret[i]);
    //    i++;
    //}
#endif

    dbus_g_method_return(context, ret);

    //[pool release];
#ifdef DEBUG
    puts("--- Invoke END ---");
#endif
}
