/*
    $Id: CSkypeAPI.h 77 2009-05-29 03:15:54Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __CSKYPEAPI_H__
#define __CSKYPEAPI_H__

#include <pthread.h>
#include <glib.h>
#include <dbus/dbus-glib.h>

#include <string>
#include <map>
#include <deque>

#include "skypeapi-dbusobj.h"

#define DBUS_SERVICE_SKYPE      "com.Skype.API"
#define DBUS_PATH_SKYPE_CLIENT  "/com/Skype/Client"
#define DBUS_PATH_CLIENT_SKPYE  "/com/Skype"
#define DBUS_INTERFACE_SKYPE    "com.Skype.API"

//#define SIZE_MAX_USERHANDLE   32
//#define SIZE_MAX_DISPLAYNAME  128
#define SIZE_MAX_USERLIST     256
#define SIZE_MAX_CALLSLIST    256
#define SIZE_ENUM_CALL_STAT   9
#define SIZE_ENUM_CALL_TYPE   4

class CSkypeAPIClient;

class CSkypeAPI {
public:
    typedef enum {
        callStat_UNDEF=-1,
        callStat_RINGING,
        callStat_ROUTING,
        callStat_FAILED,
        callStat_INPROGRESS,
        callStat_FINISHED,
        callStat_MISSED,
        callStat_REFUSED,
        callStat_BUSY,
        callStat_CANCELLED
    } callStat_t;

    static const char *callStat_str[];

    typedef enum {
        callType_UNDEF=-1,
        callType_INCOMING_PSTN,
        callType_INCOMING_P2P,
        callType_OUTGOING_PSTN,
        callType_OUTGOING_P2P
    } callType_t;

    static const char *callType_str[];

    typedef struct {
        std::string handle;
        bool online;
        time_t timestamp;
    } callsList_struct;

private:
    typedef struct {    
        const char *command;
        char **reply;
        GError **error;
        DBusGProxy *proxy;
        gboolean ret;
        bool conditionMet;
        pthread_cond_t *condition_cond;
        pthread_mutex_t *condition_mutex;
    } proxycall_struct;

    typedef struct {
        gpointer obj;
        bool conditionMet;
        pthread_cond_t *condition_cond;
        pthread_mutex_t *condition_mutex;
    } objectunref_struct;

    DBusGConnection *connection;
    DBusGProxy *proxy;
    DBusGProxy *proxy_service;
    CSkypeAPIClient *owner;
    bool isConnected;
    bool isRegistered;
    pthread_mutex_t synchro_mutex;
    pthread_mutex_t invoke_mutex;
    pthread_cond_t condition_cond;
    pthread_mutex_t condition_mutex;
    char ongoingCallId[16];
    std::map<std::string,std::string> contacts;
    std::map<std::string,std::string>::iterator contactsIter;
    std::map<std::string,callsList_struct> calls;
    std::map<std::string,callsList_struct>::iterator callsIter;
    gulong g_handler_id;

    char *_invoke(const char *command);
    void _receive(const char *message);
    bool doRegister();
    void setConnected(bool status, DBusGProxy *proxy);
    void setRegistered(bool status);
    void manageCall(const char *message);
    void updateContactList(void);
    void updateCallsList(void);
    pthread_mutex_t *getMutex(void);
    static void destroy_signal(GObject *sender, gpointer data);
    void proxyDestroyed(void);

    static gboolean gloop_objectUnref_call(gpointer data);
    static gboolean gloop_Invoke_call(gpointer data);

    class CSkypeConnectThread;
    class CSkypeRegisterThread;
    class CSkypeNotifyThread;
    class CSkypeReceiveThread;
    class CSkypePingThread;
    CSkypeConnectThread *threadConnect;
    CSkypeRegisterThread *threadRegister;
    CSkypeNotifyThread *threadNotify;
    CSkypeReceiveThread *threadReceive;
    CSkypePingThread *threadPing;
        
    class CSkypeConnectThread {
        typedef struct {    
            char ***name_list;
            GError **error;
            DBusGProxy *proxy;
            gboolean ret;
            bool conditionMet;
            pthread_cond_t *condition_cond;
            pthread_mutex_t *condition_mutex;
        } proxycall_struct;

        typedef struct {
            DBusGConnection *connection;
            GError **error;
            DBusGProxy **proxy;
            bool conditionMet;
            pthread_cond_t *condition_cond;
            pthread_mutex_t *condition_mutex;
        } newproxy_struct;

        CSkypeAPI *owner;
        DBusGConnection *connection;
        DBusGProxy *proxy_service;
        pthread_t thread;
        bool isRunning;
        pthread_cond_t condition_cond;
        pthread_mutex_t condition_mutex;

        void *execute(void);
        static void *threadfunc(void *ptr);
        bool lookupSkype(void);

        static gboolean gloop_ListNames_call(gpointer data);
        static gboolean gloop_newProxy_call(gpointer data);

    public:
        CSkypeConnectThread(CSkypeAPI *owner, DBusGConnection *connection, DBusGProxy *proxy_service);
        //bool running(void);
        void terminate(void);
    };

    class CSkypeRegisterThread {
        CSkypeAPI *owner;
        pthread_t thread;
        bool isRunning;

        void *execute(void);
        static void *threadfunc(void *ptr);

    public:
        CSkypeRegisterThread(CSkypeAPI *owner);
        bool running(void);
        void terminate(void);
    };

    class CSkypeNotifyThread {
        CSkypeAPI *owner;
        DBusGConnection *connection;
        GMainLoop *mainloop;
        SkypeAPIDBusObj *dbusobj;
        pthread_t thread;

        void *execute(void);
        static void *threadfunc(void *ptr);

    public:
        CSkypeNotifyThread(CSkypeAPI *owner, DBusGConnection *connection);
        void start(void);
        void terminate(void);
    };

    class CSkypeReceiveThread {
        CSkypeAPI *owner;
        std::deque<char *> receiveFifo;
        pthread_t thread;
        pthread_cond_t condition_cond;
        pthread_mutex_t condition_mutex;

        void *execute(void);
        static void *threadfunc(void *ptr);

    public:
        CSkypeReceiveThread(CSkypeAPI *owner);
        void start(void);
        void terminate(void);
        void receive(const char *message);
    };

    class CSkypePingThread {
        CSkypeAPI *owner;
        pthread_t thread;
        bool isRunning;

        void *execute(void);
        static void *threadfunc(void *ptr);

    public:
        CSkypePingThread(CSkypeAPI *owner);
        bool running(void);
        void terminate(void);
    };

public:
    CSkypeAPI(CSkypeAPIClient *owner);
    ~CSkypeAPI();
    void start();
    char *invoke(const char *command);
    void notify(const char *message);

    pthread_mutex_t *getDBusMutex(void);
    void *getOwner(void);

    // User functions
    char *getCallerName(const char *CallId);
    callType_t getCallType(const char *callId);
    callStat_t getCallStat(const char *callId);
    time_t getCallTimestamp(const char*);
    void initiateCall(const char *target);
    void initiateCallByName(const char *target);
    void acceptCall(void);
    void hangupCall(void);
    const char *getContactsFirst(void);
    const char *getContactsNext(void);
    const char *getContactsPrevious(void);
    callsList_struct *getCallsFirst(void);
    callsList_struct *getCallsNext(void);
    callsList_struct *getCallsPrevious(void);
};

#endif
