/*
    $Id: CSkypeConnectThread.cc 71 2009-05-27 14:54:47Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <unistd.h>
#include <string.h>
#include <glib.h>
#include <dbus/dbus-glib.h>
#include <errno.h>

#include "CLog.h"
#include "CSkypeAPI.h"

CSkypeAPI::CSkypeConnectThread::CSkypeConnectThread(CSkypeAPI *owner, DBusGConnection *connection, DBusGProxy *proxy_service) {
    this->owner = owner;
    this->connection = connection;
    this->proxy_service = proxy_service;
    
    pthread_mutex_init(&this->condition_mutex, NULL);
    pthread_cond_init(&this->condition_cond, NULL);
    
    if (0 != pthread_create( &this->thread, NULL, threadfunc, this)) {
        //this->isRunning = false;
        CLog::Log(LOGLEVEL_ERROR, "Create ConnectThread failed");
    }
    //else
        //this->isRunning = true;
}

void *CSkypeAPI::CSkypeConnectThread::threadfunc(void *ptr) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread::threadfunc");
    return ((CSkypeConnectThread*)ptr)->execute();
}

void *CSkypeAPI::CSkypeConnectThread::execute() {
    DBusGProxy *proxy;
    GError *error;
    pthread_mutex_t *synchro_mutex = this->owner->getMutex();

    newproxy_struct callstruct;

    callstruct.error = &error;
    callstruct.proxy = &proxy;
    callstruct.connection = this->connection;
    callstruct.conditionMet = false;
    callstruct.condition_cond = &this->condition_cond;
    callstruct.condition_mutex = &this->condition_mutex;

    while(1) {
        pthread_testcancel();
        
        if (!lookupSkype())
            goto next;

        error = NULL;

        pthread_mutex_lock( &this->condition_mutex );
        pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, &this->condition_mutex);

        /* Call proxy creation for skype, wait for reply */
        g_idle_add(gloop_newProxy_call, &callstruct);

        while (!callstruct.conditionMet) {
            CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread::threadfunc: pthread_cond_wait...");
            pthread_cond_wait( &this->condition_cond, &this->condition_mutex );
            CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread::threadfunc: conditionMet = %d", callstruct.conditionMet);
        }

        //pthread_mutex_unlock( &this->condition_mutex );
        pthread_cleanup_pop(1);
    
        if (proxy) {
            pthread_mutex_lock(synchro_mutex);
            pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, synchro_mutex);
            CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread calls setConnected(true)");
            this->owner->setConnected(true, proxy);
            //pthread_mutex_unlock(synchro_mutex);
            pthread_cleanup_pop(1);
            //this->isRunning = false;
            CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread EXITING normally.");         
            pthread_exit(NULL); 
        }

        CLog::Log(LOGLEVEL_ERROR, "CSkypeConnectThread: Couldn't connect to the Skype Service: %s", error->message);
        g_error_free(error);

next:
        sleep(1);
    }
}

void CSkypeAPI::CSkypeConnectThread::terminate(void) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread::terminate");
    pthread_cancel(this->thread);
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread canceled.");
    pthread_cond_signal(&this->condition_cond);
    pthread_join(this->thread, NULL);
    //this->isRunning = false;
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread finished.");
}

//bool CSkypeAPI::CSkypeConnectThread::running() {
//    return isRunning;
//}

bool CSkypeAPI::CSkypeConnectThread::lookupSkype() {
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread::lookupSkype");

    GError *error;
    bool found;
    char **name_list;
    char **name_list_ptr;
    proxycall_struct callstruct;

    callstruct.error = &error;
    callstruct.name_list = &name_list;
    callstruct.proxy = proxy_service;
    callstruct.conditionMet = false;
    callstruct.condition_cond = &this->condition_cond;
    callstruct.condition_mutex = &this->condition_mutex;

    error = NULL;

    pthread_mutex_lock( &this->condition_mutex );
    pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, &this->condition_mutex);

    /* Call ListNames method, wait for reply */
    g_idle_add(gloop_ListNames_call, &callstruct);

    while (!callstruct.conditionMet) {
        CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread::lookupSkype: pthread_cond_wait...");
        pthread_cond_wait( &this->condition_cond, &this->condition_mutex );
        CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread::lookupSkype: conditionMet = %d", callstruct.conditionMet);
    }

    //pthread_mutex_unlock( &this->condition_mutex );
    pthread_cleanup_pop(1);

    if (!callstruct.ret) {
        CLog::Log(LOGLEVEL_ERROR, "CSkypeConnectThread: prox call to 'ListNames' failed: %s", error->message);
        g_error_free(error);

        return false;
    }

    // lookup Skype
    found = false;
    for (name_list_ptr = name_list; *name_list_ptr; name_list_ptr++) {
        if (0 == strcmp("com.Skype.API", *name_list_ptr)) {
            CLog::Log(LOGLEVEL_INFO, "Skype found!");
            found = true;
            break;
        }
    }

    g_strfreev(name_list);
    return found;
}

gboolean CSkypeAPI::CSkypeConnectThread::gloop_ListNames_call(gpointer data) {
    proxycall_struct *callstruct =  (proxycall_struct *)data;

    int ret = pthread_mutex_trylock(callstruct->condition_mutex);
    
    if (ret == EBUSY) {
        CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread::gloop_ListNames_call condition_mutex BUSY");
        return TRUE; // do not unregester = retry later
    }

    if ( ret != 0) {
        CLog::Log(LOGLEVEL_ERROR, ret, "CSkypeConnectThread::gloop_ListNames_call");
        return FALSE;
    }

    callstruct->ret = dbus_g_proxy_call(callstruct->proxy, "ListNames", callstruct->error, G_TYPE_INVALID,
                          G_TYPE_STRV, callstruct->name_list, G_TYPE_INVALID);

    callstruct->conditionMet = true;
    pthread_cond_signal(callstruct->condition_cond);
    pthread_mutex_unlock(callstruct->condition_mutex);

    return FALSE; // unregister from main loop
}

gboolean CSkypeAPI::CSkypeConnectThread::gloop_newProxy_call(gpointer data) {
    newproxy_struct *callstruct =  (newproxy_struct *)data;

    int ret = pthread_mutex_trylock(callstruct->condition_mutex);
    
    if (ret == EBUSY) {
        CLog::Log(LOGLEVEL_DEBUG, "CSkypeConnectThread::gloop_newProxy_call condition_mutex BUSY");
        return TRUE; // do not unregester = retry later
    }

    if ( ret != 0) {
        CLog::Log(LOGLEVEL_ERROR, ret, "CSkypeConnectThread::gloop_newProxy_call");
        return FALSE;
    }

    /* Create a proxy object for skype */
    *callstruct->proxy = dbus_g_proxy_new_for_name_owner(callstruct->connection,
                                                  DBUS_SERVICE_SKYPE,
                                                  DBUS_PATH_CLIENT_SKPYE,
                                                  DBUS_INTERFACE_SKYPE,
                                                  callstruct->error);

    callstruct->conditionMet = true;
    pthread_cond_signal(callstruct->condition_cond);
    pthread_mutex_unlock(callstruct->condition_mutex);

    return FALSE; // unregister from main loop
}
