/*
    $Id: CSkypeNotifyThread.cc 60 2009-04-05 18:32:06Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <string.h>
#include <errno.h>

#include "CLog.h"
#include "CSkypeAPI.h"
#include "skypeapi-dbusobj.h"

CSkypeAPI::CSkypeNotifyThread::CSkypeNotifyThread(CSkypeAPI *owner, DBusGConnection *connection) {
    this->owner = owner;
    this->connection = connection;
    this->dbusobj = NULL;
}

void CSkypeAPI::CSkypeNotifyThread::start() {
    if (0 != pthread_create(&this->thread, NULL, threadfunc, this))
        CLog::Log(LOGLEVEL_ERROR, "Create CSkypeNotifyThread failed: %s", strerror(errno));
}

void *CSkypeAPI::CSkypeNotifyThread::threadfunc(void *ptr) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeNotifyThread::threadfunc");
    return ((CSkypeNotifyThread*)ptr)->execute();
}

void *CSkypeAPI::CSkypeNotifyThread::execute() {

    /* Create a main loop that will dispatch callbacks. */
    this->mainloop = g_main_loop_new (NULL, FALSE);
    if (this->mainloop == NULL) {
        /* Print error and terminate. */
        CLog::Log(LOGLEVEL_ERROR, "Couldn't create GMainLoop.");
        pthread_exit(NULL);
    }

    /*Create one instance*/
    this->dbusobj = (SkypeAPIDBusObj*)g_object_new(SKYPEAPI_TYPE_DBUSOBJ, NULL);
    if(!this->dbusobj) {
        /* Print error and terminate. */
        CLog::Log(LOGLEVEL_ERROR, "CSkypeNotifyThread: Failed to create SkypeAPIDBusObj instance.");
        pthread_exit(NULL);
    }

    skypeapi_dbusobj_set_api(this->dbusobj, this->owner);

    /*Register the instance to D-Bus*/
    dbus_g_connection_register_g_object(this->connection, DBUS_PATH_SKYPE_CLIENT, G_OBJECT (this->dbusobj));

    CLog::Log(LOGLEVEL_DEBUG, "CSkypeNotifyThread: " DBUS_PATH_SKYPE_CLIENT " method 'Notify' registered.");

    /*Run the program*/
    g_main_loop_run(this->mainloop);
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeNotifyThread: calling pthread_exit()");
    pthread_exit(NULL);
}

void CSkypeAPI::CSkypeNotifyThread::terminate() {
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeNotifyThread::terminate");
    if (this->dbusobj) 
        g_object_unref(this->dbusobj);

    if (g_main_loop_is_running(this->mainloop)) {
        CLog::Log(LOGLEVEL_DEBUG, "CSkypeNotifyThread: mainloop is running.");
        //g_main_loop_unref(mainloop);
        g_main_loop_quit(this->mainloop);
    }

    if (pthread_cancel(thread))
        CLog::Log(LOGLEVEL_DEBUG, "CSkypeNotifyThread: thread_cancel() failed => thread not running.");
    else
        CLog::Log(LOGLEVEL_DEBUG, "CSkypeNotifyThread: canceled.");
    pthread_join(this->thread, NULL);
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeNotifyThread finished.");
}
