/*
    $Id: CSkypePingThread.cc 60 2009-04-05 18:32:06Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <glib.h>
#include <dbus/dbus-glib.h>

#include "CLog.h"
#include "CSkypeAPI.h"

CSkypeAPI::CSkypePingThread::CSkypePingThread(CSkypeAPI *owner) {
    this->owner = owner;
    //g_type_init();
    
    if (0 != pthread_create( &this->thread, NULL, threadfunc, this)) {
        this->isRunning = false;
        CLog::Log(LOGLEVEL_ERROR, "Create CSkypePingThread failed");
    }
    else
        this->isRunning = true;
}

void *CSkypeAPI::CSkypePingThread::threadfunc(void *ptr) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkypePingThread::threadfunc");
    return ((CSkypePingThread*)ptr)->execute();
}

void *CSkypeAPI::CSkypePingThread::execute() {

    pthread_mutex_t *synchro_mutex = this->owner->getMutex();
    char *reply;

    while(1) {
        pthread_testcancel();
        sleep(5);
        pthread_testcancel();

        reply = this->owner->invoke("GET CONNSTATUS");
        if (!reply) {
            CLog::Log(LOGLEVEL_INFO, "CSkypePingThread: Invoke returned nothing. Trying to reconnect.");            
            goto fail;
        }

        if (0 == strcmp("CONNSTATUS ONLINE", reply)) {   
            CLog::Log(LOGLEVEL_DEBUG, "CSkypePingThread: Ping status OK");
            g_free(reply);
            continue;
        }

        if ( (0 == strcmp("CONNSTATUS OFFLINE", reply))
            || (0 == strcmp("CONNSTATUS CONNECTING", reply)) ) {   
            CLog::Log(LOGLEVEL_INFO, "Sype Connecting / User logged out.");
            pthread_mutex_lock(synchro_mutex);
            pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, synchro_mutex);
            CLog::Log(LOGLEVEL_DEBUG, "CSkypePingThread calls setRegistered(false)");
            this->owner->setRegistered(false);
            //pthread_mutex_unlock(synchro_mutex);
            pthread_cleanup_pop(1);
            this->isRunning = false;
            CLog::Log(LOGLEVEL_DEBUG, "CSkypePingThread EXITING normaly.");
            g_free(reply);
            pthread_exit(NULL); 
        }

        CLog::Log(LOGLEVEL_ERROR, "CSkypePingThread: GET STATUS returned unknown string %s", reply);
        g_free(reply);

fail:
        pthread_mutex_lock(synchro_mutex);
        pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, synchro_mutex);
        CLog::Log(LOGLEVEL_DEBUG, "CSkypePingThread calls setConnected(false)");
        this->owner->setConnected(false, NULL);
        //pthread_mutex_unlock(synchro_mutex);
        pthread_cleanup_pop(1);
        isRunning = false;
        CLog::Log(LOGLEVEL_DEBUG, "CSkypePingThread EXITING normaly.");
        pthread_exit(NULL); 
    }
}

void CSkypeAPI::CSkypePingThread::terminate(void) {
    pthread_cancel(this->thread);
    CLog::Log(LOGLEVEL_DEBUG, "CSkypePingThread canceled.");
    pthread_join(this->thread, NULL);
    this->isRunning = false;
    CLog::Log(LOGLEVEL_DEBUG, "CSkypePingThread finished.");
}

bool CSkypeAPI::CSkypePingThread::running() {
    return this->isRunning;
}
