/*
    $Id: CSkypeNotifyThread.cc 60 2009-04-05 18:32:06Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <string.h>
#include <iconv.h>
#include <errno.h>

#include "CLog.h"
#include "CSkypeAPI.h"

CSkypeAPI::CSkypeReceiveThread::CSkypeReceiveThread(CSkypeAPI *owner) {
    this->owner = owner;

    pthread_mutex_init(&condition_mutex, NULL);
    pthread_cond_init(&condition_cond, NULL);
}

void CSkypeAPI::CSkypeReceiveThread::start() {
    if (0 != pthread_create(&this->thread, NULL, threadfunc, this))
        CLog::Log(LOGLEVEL_ERROR, "Create CSkypeReceiveThread failed: %s", strerror(errno));
}

void *CSkypeAPI::CSkypeReceiveThread::threadfunc(void *ptr) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeReceiveThread::threadfunc");
    return ((CSkypeReceiveThread*)ptr)->execute();
}

void *CSkypeAPI::CSkypeReceiveThread::execute() {
    char *message;

    while (1) {
        pthread_testcancel();

        pthread_mutex_lock(&this->condition_mutex);
        if (this->receiveFifo.empty()) {
            CLog::Log(LOGLEVEL_DEBUG,"CSkypeReceiveThread::execute sendfifo empty cond_wait...");
            pthread_cond_wait(&this->condition_cond, &this->condition_mutex);
            CLog::Log(LOGLEVEL_DEBUG,"CSkypeReceiveThread::execute end cond_wait");
        }

        pthread_testcancel();

        if (!this->receiveFifo.empty()) {
            message = this->receiveFifo.front();
            this->receiveFifo.pop_front();
        }
        else {
            message = NULL;
        }
        pthread_mutex_unlock(&this->condition_mutex);

        if (message) {
            this->owner->_receive(message);
            delete [] message;
        }
    }
}

void CSkypeAPI::CSkypeReceiveThread::receive(const char *message) {
    size_t size, inlen, outlen;
    char *buf, *outmarker;
    char *inmarker;
    iconv_t cd;

    if ((size = strlen(message)) < 1)
        return;

    inlen = outlen = size;
    
    inmarker =(char *)message;
    buf= outmarker = new char[size+1];
    
    if ((cd = iconv_open("ISO-8859-1", "UTF-8")) == (iconv_t)(-1)) {
        CLog::Log(LOGLEVEL_ERROR, errno, "iconv_open: error");
        return;
    }
    
    // Convert every character within input string.
    while (inlen > 0) {
        if (iconv(cd, &inmarker, &inlen, &outmarker, &outlen) == (size_t)(-1)) {
            CLog::Log(LOGLEVEL_ERROR, errno, "iconv_string: conversion error");
            iconv_close(cd);
            return;
        }
    }
    *outmarker = '\0';
    iconv_close(cd);

    pthread_mutex_lock(&condition_mutex);
    this->receiveFifo.push_back(buf);
    pthread_cond_signal(&condition_cond);
    pthread_mutex_unlock(&condition_mutex);
}

void CSkypeAPI::CSkypeReceiveThread::terminate() {
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeReceiveThread::terminate");

    if (pthread_cancel(thread))
        CLog::Log(LOGLEVEL_DEBUG, "CSkypeReceiveThread: thread_cancel() failed => thread not running.");
    else
        CLog::Log(LOGLEVEL_DEBUG, "CSkypeReceiveThread: canceled.");
    pthread_cond_signal(&condition_cond);
    pthread_join(this->thread, NULL);
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeReceiveThread finished.");
}
