/*
    $Id: CSkypeRegisterThread.cc 60 2009-04-05 18:32:06Z twimmer $; This file is part of:

    skiputler. Connects Topcom "Butler 4012 USB VoIP" with Skype on Linux.
    Copyright (C) 2009  Till Wimmer skiputler@tonarchiv.ch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <glib.h>
#include <dbus/dbus-glib.h>

#include "CLog.h"
#include "CSkypeAPI.h"

CSkypeAPI::CSkypeRegisterThread::CSkypeRegisterThread(CSkypeAPI *owner) {
    this->owner = owner;
    //g_type_init();
    
    if (0 != pthread_create( &this->thread, NULL, threadfunc, this)) {
        this->isRunning = false;
        CLog::Log(LOGLEVEL_ERROR, "Create CSkypeRegisterThread failed");
    }
    else
        this->isRunning = true;
}

void *CSkypeAPI::CSkypeRegisterThread::threadfunc(void *ptr) {
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeRegisterThread::threadfunc");
    return ((CSkypeRegisterThread*)ptr)->execute();
}

void *CSkypeAPI::CSkypeRegisterThread::execute() {

    pthread_mutex_t *synchro_mutex = owner->getMutex();
    char *reply;
    int sleeptime;
    bool registered = false;

    while(1) {
        pthread_testcancel();
        sleeptime = 1;

        if (!registered) {
            reply = this->owner->invoke("NAME Butler");
            if (!reply) {
                CLog::Log(LOGLEVEL_INFO, "CSkypeRegisterThread: Invoke returned nothing. Trying to reconnect.");            
                pthread_mutex_lock(synchro_mutex);
                pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, synchro_mutex);
                CLog::Log(LOGLEVEL_DEBUG, "CSkypeRegisterThread calls setConnected(false)");
                this->owner->setConnected(false, NULL);
                //pthread_mutex_unlock(synchro_mutex);
                pthread_cleanup_pop(1);
                this->isRunning = false;
                CLog::Log(LOGLEVEL_DEBUG, "CSkypeRegisterThread EXITING normally.");         
                pthread_exit(NULL);
            }

            if (0 == strcmp("CONNSTATUS OFFLINE", reply)) {   
                CLog::Log(LOGLEVEL_INFO, "Skype: No user logged in.");
                g_free(reply);
                sleeptime = 1;
                goto next;
            }

            if (0 != strcmp("OK", reply)) {
                CLog::Log(LOGLEVEL_ERROR, "CSkypeRegisterThread: Wrong reply: %s", reply);
                g_free(reply);
                pthread_exit(NULL); // TODO
            }

            g_free(reply);
            reply = this->owner->invoke("PROTOCOL 7");
            if (!reply) {
                CLog::Log(LOGLEVEL_INFO, "CSkypeRegisterThread: Invoke returned nothing. Trying to reconnect.");            
                pthread_mutex_lock(synchro_mutex);
                pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, synchro_mutex);
                CLog::Log(LOGLEVEL_DEBUG, "CSkypeRegisterThread calls setConnected(false)");
                this->owner->setConnected(false, NULL);
                //pthread_mutex_unlock(synchro_mutex);
                pthread_cleanup_pop(1);
                this->isRunning = false;
                CLog::Log(LOGLEVEL_DEBUG, "CSkypeRegisterThread EXITING normally.");         
                pthread_exit(NULL);
            }

            if (0 != strcmp("PROTOCOL 7", reply)) {
                CLog::Log(LOGLEVEL_ERROR, "CSkypeRegisterThread: Wrong reply: %s", reply);
                g_free(reply);
                pthread_exit(NULL); //TODO
            }
            registered = true;
            g_free(reply);
        }

        pthread_testcancel();
        sleep(5);
 
       reply = this->owner->invoke("GET CONNSTATUS");
        if (!reply) {
            CLog::Log(LOGLEVEL_INFO, "CSkypeRegisterThread: Invoke returned nothing. Trying to reconnect.");            
            pthread_mutex_lock(synchro_mutex);
            pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, synchro_mutex);
            CLog::Log(LOGLEVEL_DEBUG, "CSkypeRegisterThread calls setConnected(false)");
            this->owner->setConnected(false, NULL);
            //pthread_mutex_unlock(synchro_mutex);
            pthread_cleanup_pop(1);
            this->isRunning = false;
            CLog::Log(LOGLEVEL_DEBUG, "CSkypeRegisterThread EXITING normally.");         
            pthread_exit(NULL);
        }

        if ( (0 == strcmp("CONNSTATUS OFFLINE", reply))
            || (0 == strncmp("ERROR", reply, 5)) ) {   
            CLog::Log(LOGLEVEL_INFO, "Skype: No user logged in.");
            g_free(reply);
            registered = false;
            sleeptime = 1;
            goto next;
        }

        if (0 != strcmp("CONNSTATUS ONLINE", reply)) {
            CLog::Log(LOGLEVEL_INFO, "CSkypeRegisterThread: Skype offline", reply);
            g_free(reply);
            sleeptime = 5;
            goto next;
        }

        g_free(reply);

        pthread_mutex_lock(synchro_mutex);
        pthread_cleanup_push((void (*)(void*))pthread_mutex_unlock, synchro_mutex);
        CLog::Log(LOGLEVEL_DEBUG, "CSkypeRegisterThread calls setRegistered(true)");
        this->owner->setRegistered(true);
        //pthread_mutex_unlock(synchro_mutex);
        pthread_cleanup_pop(1);

        this->isRunning = false;
        CLog::Log(LOGLEVEL_DEBUG, "CSkypeRegisterThread EXITING normally.");         
        pthread_exit(NULL); 

next:
        sleep(sleeptime);
    }
}

void CSkypeAPI::CSkypeRegisterThread::terminate(void) {
    pthread_cancel(this->thread);
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeRegisterThread canceled.");
    pthread_join(this->thread, NULL);
    this->isRunning = false;
    CLog::Log(LOGLEVEL_DEBUG, "CSkypeRegisterThread finished.");
}

bool CSkypeAPI::CSkypeRegisterThread::running() {
    return this->isRunning;
}
